from django.db import models
from .dragon_test_case import DragonTestCase
from .models import SDModel, ParentModel, ChildModel
from swampdragon.serializers.model_serializer import ModelSerializer


class FooOne2OneOM(SDModel):
    name = models.CharField(max_length=20)


class BarOne2OneOM(SDModel):
    foo = models.OneToOneField(FooOne2OneOM)
    number = models.IntegerField()


class FooM2MOM(SDModel):
    name = models.CharField(max_length=20)


class BarM2MOM(SDModel):
    foos = models.ManyToManyField(FooM2MOM)
    number = models.IntegerField()


class FooM2MSerializer(ModelSerializer):
    barm2mom_set = 'BarM2MOMSerializer'

    class Meta:
        model = FooM2MOM
        update_fields = ('name', 'barm2mom_set')


class BarM2MOMSerializer(ModelSerializer):
    foos = FooM2MSerializer

    class Meta:
        model = BarM2MOM
        update_fields = ('number', 'foos')


class FooSerializerOM(ModelSerializer):
    barone2oneom = 'BarSerializerOM'

    class Meta:
        model = FooOne2OneOM
        update_fields = ('name', 'barone2oneom')


class BarSerializerOM(ModelSerializer):
    foo = FooSerializerOM

    class Meta:
        model = BarOne2OneOM


class ParentSerializerOM(ModelSerializer):
    childmodel_set = 'ChildSerializerOM'

    class Meta:
        model = ParentModel
        publish_fields = ('text', 'childmodel_set')


class ChildSerializerOM(ModelSerializer):
    parent = ParentSerializerOM

    class Meta:
        model = ChildModel
        publish_fields = ('number', 'parent')


class TestObjectMap(DragonTestCase):
    def test_get_object_map_for_one2one(self):
        om = FooSerializerOM.get_object_map()
        for m in om:
            self.assertFalse(m['is_collection'])

    def test_get_object_map_for_reverse_one2one(self):
        om = BarSerializerOM.get_object_map()
        for m in om:
            self.assertFalse(m['is_collection'])

    def test_get_object_map_for_fk(self):
        om = ChildSerializerOM.get_object_map()
        self.assertEqual(len(om), 2)

    def test_get_object_map_for_reverse_fk(self):
        om = ParentSerializerOM.get_object_map()
        self.assertEqual(len(om), 2)

    def test_get_object_map_form_m2m(self):
        om = FooM2MSerializer.get_object_map()
        self.assertEqual(len(om), 2)



from .dragon_test_case import DragonTestCase
from ..route_handler import BaseRouter, LOGIN_REQUIRED, SUCCESS, ERROR
from ..permissions import login_required, LoginRequired, RoutePermission


class TestRouterDecorated(BaseRouter):
    """
    A router with a function decorated
    """
    valid_verbs = ['do_something']

    @login_required
    def do_something(self, **kwargs):
        self.send('all good')


class TestRouter(BaseRouter):
    """
    A router with all functions requiring login
    """
    valid_verbs = ['do_something']
    permission_classes = [LoginRequired()]

    def do_something(self, **kwargs):
        self.send('all good')


class TestRouterSpecific(BaseRouter):
    """
    A router where only one verb requires a signed in user
    """
    valid_verbs = ['free_for_all', 'need_login']
    permission_classes = [LoginRequired(verbs=['need_login'])]

    def free_for_all(self, **kwargs):
        self.send('all good')

    def need_login(self, **kwargs):
        self.send('all good')


class CustomPermission(RoutePermission):
    """
    A custom permission requiring 'ok' to be sent to the handler
    """
    def test_permission(self, handler, verb, **kwargs):
        if 'ok' not in kwargs:
            return self.permission_failed(handler)
        return True

    def permission_failed(self, handler):
        handler.send_error({'ok': ['ok missing']})
        return False


class CustomPermissionRouter(BaseRouter):
    """
    A router with a custom permission
    """
    valid_verbs = ['permission_required']
    permission_classes = [CustomPermission()]

    def permission_required(self, **kwargs):
        self.send('everything is okay')


class BrokenPermission(RoutePermission):
    """
    This permission is missing both required functions of a permission
    *  test_permission
    *  permission_failed
    """
    pass


class BrokenPermissionRouter(BaseRouter):
    valid_verbs = ['do_something']
    permission_classes = [BrokenPermission()]

    def do_something(self, **kwargs):
        pass


class HalfAPermission(RoutePermission):
    def test_permission(self, handler, verb, **kwargs):
        return self.permission_failed(handler)


class HalfAPermissionRouter(BaseRouter):
    valid_verbs = ['do_something']
    permission_classes = [HalfAPermission()]

    def do_something(self, **kwargs):
        pass


class TestPermissions(DragonTestCase):
    def test_login_required_decorator(self):
        router = TestRouterDecorated(self.connection)
        router.handle({'verb': 'do_something'})
        self.assertEqual(self.connection.last_message['context']['state'], LOGIN_REQUIRED)

    def test_login_required_decorator_with_user(self):
        self.connection.user = {'user': 'test user'}
        router = TestRouterDecorated(self.connection)
        router.handle({'verb': 'do_something'})
        self.assertEqual(self.connection.last_message['context']['state'], SUCCESS)

    def test_login_required(self):
        router = TestRouter(self.connection)
        router.handle({'verb': 'do_something'})
        self.assertEqual(self.connection.last_message['context']['state'], LOGIN_REQUIRED)

    def test_login_required_on_one_verb(self):
        router = TestRouterSpecific(self.connection)
        router.handle({'verb': 'free_for_all'})
        self.assertEqual(self.connection.last_message['context']['state'], SUCCESS)

        router.handle({'verb': 'need_login'})
        self.assertEqual(self.connection.last_message['context']['state'], LOGIN_REQUIRED)

    def test_fail_custom_permission(self):
        router = CustomPermissionRouter(self.connection)
        router.handle({'verb': 'permission_required'})
        self.assertEqual(self.connection.last_message['context']['state'], ERROR)

    def test_pass_custom_permission(self):
        router = CustomPermissionRouter(self.connection)
        router.handle({'verb': 'permission_required', 'args': {'ok': True}})
        self.assertEqual(self.connection.last_message['context']['state'], SUCCESS)

    def test_broken_permission(self):
        router = BrokenPermissionRouter(self.connection)
        with self.assertRaises(NotImplementedError):
            router.handle({'verb': 'do_something'})

    def test_permission_missing_permission_failed(self):
        router = HalfAPermissionRouter(self.connection)
        with self.assertRaises(NotImplementedError):
            router.handle({'verb': 'do_something'})



from .channel_utils import make_safe


class PUBACTIONS:
    created = 'created'
    updated = 'updated'
    deleted = 'deleted'


class BaseProvider(object):
    def _construct_channel(self, base_channel, **channel_filter):
        sorted_filter_keys = sorted(channel_filter, key=channel_filter.get)
        filter_string = '|'.join(['{}:{}'.format(k, make_safe(channel_filter[k])) for k in sorted_filter_keys])
        complete_channel = '{}|{}'.format(base_channel, filter_string)
        return complete_channel

    def get_channel(self, base_channel, **channel_filter):
        return self._construct_channel(base_channel, **channel_filter)



def format_message(data, context, channel_setup=None):
    message = dict({'data': data})
    message['context'] = context
    if channel_setup:
        message['channel_data'] = channel_setup
    return message



cache = {}


def store_data(data, key, ttl):
    cache[key] = data


def get_data(key):
    return cache[key]


def has_data(key):
    return cache.get(key) is not None


def remove(key):
    if has_data(key):
        cache.pop(key)



from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TextModel, ParentModel, ChildModel


class TextModelSerializer(ModelSerializer):
    class Meta:
        model = TextModel
        publish_fields = ('text', 'custom_field')

    def serialize_custom_field(self, obj):
        return 'data'


class CustomTextModelSerializer(ModelSerializer):
    class Meta:
        model = TextModel

    def serialize_foo(self, obj):
        return 'bar'


class ChildSerializer(ModelSerializer):
    class Meta:
        model = ChildModel


class RelatedSerializer(ModelSerializer):
    childmodel_set = 'ChildSerializer'

    class Meta:
        model = ParentModel

    def serialize_childmodel_set(self, obj, serializer):
        return 'c {}'.format(serializer(instance=obj.childmodel_set.first()).serialize()['id'])


class TestModelSerializer(DragonTestCase):
    def test_serialize_custom_field(self):
        text_model = TextModel.objects.create(text='hello world')
        result = TextModelSerializer(instance=text_model).serialize()
        self.assertEqual(result['custom_field'], 'data')

    def test_serialize_custom_field_not_in_publish_fields(self):
        text_model = TextModel.objects.create(text='hello world')
        result = CustomTextModelSerializer(instance=text_model).serialize()
        self.assertEqual(result['foo'], 'bar')

    def test_serialize_custom_related(self):
        parent = ParentModel.objects.create(name='foo')
        child = ChildModel.objects.create(number=19, parent=parent)
        data = RelatedSerializer(instance=parent).serialize()
        self.assertEqual(data['childmodel_set'], 'c {}'.format(child.pk))



from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import TextModel, TwoFieldModel, ChildModel, ParentModel


class TextModelSerializer(ModelSerializer):
    class Meta:
        model = TextModel
        publish_fields = ('text', )
        base_channel = 'custom_base_channel'


class AllFieldSerializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class ChildModelSerializer(ModelSerializer):
    class Meta:
        model = ChildModel


class TwoFieldModelSerializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class TestModelSerializer(DragonTestCase):
    def test_serialize_model(self):
        text_model = TextModel.objects.create(text='hello world')
        serializer = TextModelSerializer(instance=text_model)
        data = serializer.serialize()
        self.assertEqual(text_model.pk, data['id'])
        self.assertEqual(text_model.text, data['text'])

    def test_serializer_without_instance(self):
        """
        Serializing without instance should return None
        """
        expected = None
        actual = TextModelSerializer().serialize()
        self.assertEqual(expected, actual)

    def test_serialize_custom_base_channel(self):
        self.assertEqual(TextModelSerializer.get_base_channel(), 'custom_base_channel|')

    def test_serializer_auto_set_publish_fields(self):
        tfm = TwoFieldModel.objects.create(text='foo', number=123)
        ser = AllFieldSerializer(instance=tfm)
        data = ser.serialize()
        self.assertEqual(tfm.text, data['text'])
        self.assertEqual(tfm.number, data['number'])

    def test_serialize_include_related_as_ids(self):
        """
        Include related fields as ids, with no publish_fields set
        """
        parent = ParentModel.objects.create(name='parent', pk=99)
        child = ChildModel.objects.create(number=8, parent=parent)
        ser = ChildModelSerializer(instance=child)
        data = ser.serialize()
        self.assertEqual(data['parent'], parent.pk)

    def test_serialize_individual_field(self):
        """
        Assigning a list of fields should not use the default fields.
        """
        tfm = TwoFieldModel.objects.create(text='hello', number='123')
        ser = TwoFieldModelSerializer(instance=tfm)
        data = ser.serialize()
        self.assertIn('text', ser.serialize())
        self.assertIn('number', ser.serialize())
        self.assertNotIn('text', ser.serialize(['number']))
        self.assertIn('number', ser.serialize(['number']))



from django.conf import settings
from django.utils.importlib import import_module
from tornado import web, ioloop
from sockjs.tornado import SockJSRouter
from swampdragon import discover_routes, load_field_deserializers
from swampdragon.settings_provider import SettingsHandler
import os
import sys


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "<project>.settings")

if __name__ == '__main__':
    args = sys.argv
    HOST = '127.0.0.1'
    PORT = 9999

    if len(args) > 1:
        host_port = args[1]
        HOST = host_port.split(':')[0]
        PORT = host_port.split(':')[1]
    routers = []
    for sockjs_class in settings.SOCKJS_CLASSES:
        module_name, cls_name = sockjs_class[0].rsplit('.', 1)
        module = import_module(module_name)
        cls = getattr(module, cls_name)
        channel = sockjs_class[1]
        routers.append(SockJSRouter(cls, channel))
        print('Channel {}'.format(channel))

    app_settings = {
        'debug': settings.DEBUG,
    }

    urls = discover_routes()
    for router in routers:
        urls += router.urls
    urls.append(('/settings.js$', SettingsHandler))

    load_field_deserializers()

    app = web.Application(urls, **app_settings)
    app.listen(PORT, address=HOST, no_keep_alive=False)
    print('Running SwampDragon on {}:{}'.format(HOST, PORT))
    try:
        iol = ioloop.IOLoop.instance()
        iol.start()
    except KeyboardInterrupt:
        # so you don't think you erred when ^C'ing out
        pass



from ..route_handler import BaseRouter, ERROR, SUCCESS, LOGIN_REQUIRED
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_send(self):
        message = 'today is a good day'
        self.router.send(message)
        self.assertEqual(self.connection.last_message['data'], message)
        self.assertEqual(self.connection.last_message['context']['state'], SUCCESS)

    def test_send_error(self):
        error = "it's gone wrong"
        self.router.send_error(error)
        self.assertEqual(self.connection.last_message['data'], error)
        self.assertEqual(self.connection.last_message['context']['state'], ERROR)

    def test_login_required(self):
        self.router.send_login_required()
        self.assertEqual(self.connection.last_message['context']['state'], LOGIN_REQUIRED)



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'

    def get_subscription_channels(self, **kwargs):
        channel = 'foo-{}'.format(kwargs['value'])
        return [channel]


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_subscribe(self):
        data = {
            'value': 'bar',
            'channel': 'client-channel'
        }
        self.router.subscribe(**data)
        self.assertEqual(self.connection.last_message['data'], 'subscribed')
        self.assertListEqual(self.connection.last_message['channel_data']['remote_channels'], ['foo-bar'])

    def test_unsubscribe(self):
        data = {
            'value': 'bar',
            'channel': 'client-channel'
        }
        self.router.unsubscribe(**data)
        self.assertEqual(self.connection.last_message['data'], 'unsubscribed')
        self.assertListEqual(self.connection.last_message['channel_data']['remote_channels'], ['foo-bar'])



from django.db import models
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import ParentModel, ChildModel, SDModel


class Address(SDModel):
    name = models.CharField(max_length=100)


class AddressBook(SDModel):
    owner = models.CharField(max_length=100)
    address = models.ManyToManyField(Address)


class AddressBookSerializer(ModelSerializer):
    class Meta:
        model = AddressBook
        publish_fields = ('owner', 'address')


class AddressSerializer(ModelSerializer):
    class Meta:
        model = Address
        publish_fields = ('name', 'addressbook_set')


class ExtendedAddressSerializer(AddressSerializer):
    addressbook_set = AddressBookSerializer


class ExtendedAddressBookSerializer(AddressBookSerializer):
    address = AddressSerializer


class TestModelSerializer(DragonTestCase):
    def test_serialize_m2m(self):
        address = Address.objects.create(name='benedicte')
        address_book = AddressBook.objects.create(owner='veronica')

        address_book.address.add(address)

        ser = ExtendedAddressBookSerializer(instance=address_book)

        data = ser.serialize()
        self.assertEqual(data['address'][0]['name'], address.name)


    def test_serialize_reverse_m2m(self):
        address = Address.objects.create(name='benedicte')
        address_book = AddressBook.objects.create(owner='veronica')
        address_book.address.add(address)

        ser = ExtendedAddressSerializer(instance=address)
        data = ser.serialize()
        self.assertEqual(data['addressbook_set'][0]['owner'], address_book.owner)


    def test_serialize_m2m_without_serializer(self):
        """
        Serialize an m2m relation without a serializer
        should return a list ids
        """
        address_a = Address.objects.create(name='benedicte')
        address_b = Address.objects.create(name='jonas')
        address_book = AddressBook.objects.create(owner='veronica')

        address_book.address.add(address_a)
        address_book.address.add(address_b)

        ser = AddressBookSerializer(instance=address_book)
        data = ser.serialize()
        self.assertListEqual(list(data['address']), [address_a.pk, address_b.pk])

    def test_serialize_reverse_m2m_without_serializer(self):
        """
        Serialize a reverse m2m relation without a serializer
        should return a list ids
        """
        address = Address.objects.create(name='benedicte')
        address_book = AddressBook.objects.create(owner='veronica')
        address_book.address.add(address)

        ser = AddressSerializer(instance=address)
        data = ser.serialize()
        self.assertListEqual(list(data['addressbook_set']), [address_book.pk])



# from django.db import models
#
#
# class ParentModel(models.Model):
#     name = models.CharField(max_length=100)
#     age = models.PositiveIntegerField()
#
#
# class ChildModel(models.Model):
#     parent = models.ForeignKey(ParentModel, related_name='children')
#     name = models.CharField(max_length=100)
#
#
# class SubChildModel(models.Model):
#     child = models.ForeignKey(ChildModel, related_name='subchildren')
#     name = models.CharField(max_length=100)



import json
import redis

redis_cli = redis.StrictRedis()


def publish(channel, message):
    redis_cli.publish(channel, json.dumps(message))


def _get_channels_from_redis(base_channel):
    channels = redis_cli.execute_command('PUBSUB', 'channels', '{}*'.format(base_channel))
    return [c.decode() for c in channels]


def get_channels(base_channel):
    return _get_channels_from_redis(base_channel)



from .dragon_test_case import DragonTestCase
from .models import TextModel, ParentModel, ChildModel
from .. import model_tools


class TestModelTools(DragonTestCase):
    def test_get_property(self):
        text_model = TextModel.objects.create(text='test')
        val = model_tools.get_property(text_model, 'text')
        self.assertEqual(val, text_model.text)

    def test_get_related_property(self):
        parent = ParentModel.objects.create(name='a parent')
        child = ChildModel.objects.create(number=12, parent=parent)
        val = model_tools.get_property(child, 'parent__name')
        self.assertEqual(val, parent.name)

    def test_get_invalid_related_property(self):
        parent = ParentModel.objects.create(name='a parent')
        child = ChildModel.objects.create(number=12, parent=parent)
        self.assertIsNone(model_tools.get_property(parent, 'parent__invalid_property__p'))

    def test_string_to_list(self):
        data_string = '[a,b,c]'
        expected = ['a', 'b', 'c']
        actual = model_tools.string_to_list(data_string)
        self.assertListEqual(expected, actual)

    def test_get_model(self):
        parent_model = 'tests.ParentModel'
        model = model_tools.get_model(parent_model)
        self.assertEqual(model, ParentModel)



from django.conf import settings
from django.utils.importlib import import_module
from .redis_session_store import RedisSessionStore

session_store = None


def get_session_store():
    global session_store

    if session_store:
        return session_store
    try:
        module_name, cls_name = settings.DRAGON_SESSION_STORE.rsplit('.', 1)
        module = import_module(module_name)
        cls = getattr(module, cls_name)
        store = cls()
        session_store = store
    except:
        return RedisSessionStore()



from django.conf import settings as django_settings


class SwampDragonSettings(object):
    def to_dict(self):
        settings = getattr(django_settings, 'SWAMP_DRAGON', {})
        settings['endpoints'] = []
        for connection, endpoint in django_settings.SOCKJS_CLASSES:
            settings['endpoints'].append(endpoint.replace('/', ''))
        return settings



from django.test import TestCase
from django.conf import settings
from django.utils.importlib import import_module
from sockjs.tornado import SockJSRouter
from tornado import web
from .. import discover_routes
from .. import route_handler
from ..pubsub_providers.subscriber_factory import get_subscription_provider
from .mock_connection import TestConnection


pub_sub = get_subscription_provider()


class DragonTestCase(TestCase):
    def __init__(self, methodName='runTest'):
        super(DragonTestCase, self).__init__(methodName)
        pub_sub._channels = []
        pub_sub._subscribers = {}
        self.connection = TestConnection()
        self.urls = discover_routes()

    def tearDown(self):
        route_handler.registered_handlers = {}
        pub_sub._channels = []
        pub_sub._subscribers = {}

    def _load_app(self):
        routers = []
        for sockjs_class in settings.SOCKJS_CLASSES:
            module_name, cls_name = sockjs_class[0].rsplit('.', 1)
            module = import_module(module_name)
            cls = getattr(module, cls_name)
            channel = sockjs_class[1]
            routers.append(SockJSRouter(cls, channel))
            print('Channel {}'.format(channel))

        app_settings = {
            'debug': settings.DEBUG,
        }

        urls = discover_routes()
        for router in routers:
            urls += router.urls

        app = web.Application(urls, **app_settings)
        return app



from django.db import models
from .dragon_test_case import DragonTestCase
from ..serializers.model_serializer import ModelSerializer
from ..serializers import serializer_tools
from swampdragon.tests.models import SDModel


class ReverseM2M(SDModel):
    number = models.IntegerField()


class M2M(SDModel):
    name = models.CharField(max_length=10)
    many = models.ManyToManyField(ReverseM2M)


class ReverseFk(SDModel):
    name = models.CharField(max_length=10)


class Fk(SDModel):
    number = models.IntegerField()
    reverse_fk = models.ForeignKey(ReverseFk)


class ReverseM2MSerializer(ModelSerializer):
    m2m_set = 'M2MSerializer'

    class Meta:
        model = ReverseM2M
        publish_fields = ('number', 'm2m_set')


class ReverseO2O(SDModel):
    bar = models.CharField(max_length=100)


class O2O(SDModel):
    foo = models.CharField(max_length=100)
    reverse_o2o = models.OneToOneField(ReverseO2O)


class M2MSerializer(ModelSerializer):
    many = ReverseM2MSerializer

    class Meta:
        model = M2M


class FKSerializer(ModelSerializer):
    reverse_fk = 'ReverseFKSerializer'

    class Meta:
        model = Fk
        publish_fields = ('reverse_fk', )


class ReverseFKSerializer(ModelSerializer):
    fk_set = FKSerializer

    class Meta:
        model = ReverseFk
        publish_fields = ('fk_set', )


class O2OSerializer(ModelSerializer):
    reverse_o2o = 'ReverseO2OSerializer'

    class Meta:
        model = O2O


class ReverseO2OSerializer(ModelSerializer):
    o2o = O2OSerializer

    class Meta:
        model = ReverseO2O
        fields = ('o2o')


class TestSerializerTools(DragonTestCase):
    def test_m2m(self):
        reverse_m2m = ReverseM2M.objects.create(number=12)
        m2m = M2M.objects.create(name='test')
        m2m.many.add(reverse_m2m)
        mapping = serializer_tools.get_id_mappings(M2MSerializer(instance=m2m))
        self.assertEqual(list(mapping['many']), [reverse_m2m.pk])

    def test_reverse_m2m(self):
        reverse_m2m = ReverseM2M.objects.create(number=12)
        m2m = M2M.objects.create(name='test')
        m2m.many.add(reverse_m2m)
        mapping = serializer_tools.get_id_mappings(ReverseM2MSerializer(instance=reverse_m2m))
        self.assertEqual(list(mapping['m2m_set']), [m2m.pk])

    def test_fk(self):
        reverse_fk = ReverseFk.objects.create(name='test')
        fk = Fk.objects.create(number=99, reverse_fk=reverse_fk)
        mapping = serializer_tools.get_id_mappings(FKSerializer(instance=fk))
        self.assertEqual(mapping['reverse_fk'], reverse_fk.pk)

    def test_reverse_fk(self):
        reverse_fk = ReverseFk.objects.create(name='test')
        fk = Fk.objects.create(number=99, reverse_fk=reverse_fk)
        mapping = serializer_tools.get_id_mappings(ReverseFKSerializer(instance=reverse_fk))
        self.assertEqual(list(mapping['fk_set']), [fk.pk])

    def test_one2one(self):
        ro2o = ReverseO2O.objects.create(bar='another test')
        o2o = O2O.objects.create(foo='test', reverse_o2o=ro2o)
        mapping = serializer_tools.get_id_mappings(O2OSerializer(instance=o2o))
        self.assertEqual(mapping['reverse_o2o'], ro2o.pk)

    def test_reverse_one2one(self):
        ro2o = ReverseO2O.objects.create(bar='another test')
        o2o = O2O.objects.create(foo='test', reverse_o2o=ro2o)
        mapping = serializer_tools.get_id_mappings(ReverseO2OSerializer(instance=ro2o))
        self.assertEqual(mapping['o2o'], o2o.pk)



from django.db import models
from ..models import SelfPublishModel
from swampdragon.tests.serializers import FooSelfPubSerializer, BarSelfPubSerializer


class SDModel(models.Model):
    class Meta:
        app_label = 'tests'
        abstract = True


class TextModel(SDModel):
    text = models.CharField(max_length=100)


class TwoFieldModel(SDModel):
    text = models.CharField(max_length=50)
    number = models.IntegerField()


class ParentModel(SDModel):
    name = models.CharField(max_length=100)


class ChildModel(SDModel):
    parent = models.ForeignKey(ParentModel)
    number = models.IntegerField()


class FooSelfPub(SelfPublishModel, SDModel):
    serializer_class = FooSelfPubSerializer
    name = models.CharField(max_length=100)
    number = models.IntegerField(null=True)


class BarSelfPub(SelfPublishModel, SDModel):
    serializer_class = BarSelfPubSerializer
    date = models.DateTimeField()
    foo = models.ForeignKey(FooSelfPub, related_name='bars')



from django.core.exceptions import ValidationError
from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor, ForeignRelatedObjectsDescriptor, \
    ReverseManyRelatedObjectsDescriptor, ManyRelatedObjectsDescriptor
from swampdragon.model_tools import get_property, get_model
from swampdragon.serializers.field_serializers import serialize_field
from swampdragon.serializers.object_map import get_object_map
from swampdragon.serializers.serializer_importer import get_serializer
from swampdragon.serializers.field_deserializers import get_deserializer
from swampdragon.serializers.serializer_tools import get_serializer_relationship_field, get_id_mappings
from swampdragon.serializers.validation import ModelValidationError


class ModelSerializerMeta(object):
    def __init__(self, options):
        self.model = get_model(getattr(options, 'model'))
        self.publish_fields = getattr(options, 'publish_fields', None)

        if not self.publish_fields:
            self.publish_fields = self.get_fields(self.model)

        if isinstance(self.publish_fields, str):
            self.publish_fields = (self.publish_fields, )

        self.update_fields = getattr(options, 'update_fields', ())
        if isinstance(self.update_fields, str):
            self.update_fields = (self.update_fields, )

        self.id_field = getattr(options, 'id_field', 'pk')
        self.base_channel = getattr(options, 'base_channel', self.model._meta.model_name)

    def get_fields(self, model):
        fields = []
        for f in model._meta.get_all_field_names():
            field = model._meta.get_field_by_name(f)[0]
            if hasattr(field, 'get_accessor_name'):
                fields.append(field.get_accessor_name())
            else:
                fields.append(field.name)
        return fields


class ModelSerializer(object):
    def __init__(self, data=None, instance=None, initial=None):
        if data and not isinstance(data, dict):
            raise Exception('data needs to be a dictionary')
        self.opts = ModelSerializerMeta(self.Meta)
        self._instance = instance
        self.data = data
        self.initial = initial or {}
        self.base_fields = self._get_base_fields()
        self.m2m_fields = self._get_m2m_fields()
        self.related_fields = self._get_related_fields()
        self.errors = {}

    class Meta(object):
        pass

    @property
    def instance(self):
        return self._instance

    def _get_base_fields(self):
        return [f.name for f in self.opts.model._meta.fields]

    def _get_related_fields(self):
        return [f for f in self.opts.update_fields if f not in self.base_fields and f not in self.m2m_fields]

    def _get_m2m_fields(self):
        related_m2m = [f.get_accessor_name() for f in self.opts.model._meta.get_all_related_many_to_many_objects()]
        m2m_fields = [f.name for f in self.opts.model._meta.local_many_to_many]
        m2m = m2m_fields + related_m2m
        return [f for f in self.opts.update_fields if f in m2m]

    def deserialize(self):
        # Set initial data
        if not self._instance:
            self._instance = self.opts.model()

        for key, val in self.initial.items():
                setattr(self.instance, key, val)

        # Deserialize base fields
        for key, val in self.data.items():
            if key not in self.opts.update_fields or key not in self.base_fields:
                continue
            try:
                self.validate_field(key, val, self.data)
                self._deserialize_field(key, val)
            except ModelValidationError as err:
                self.errors.update(err.get_error_dict())


        if self.errors:
            raise ModelValidationError(errors=self.errors)

        return self.instance

    def save(self):
        self.deserialize()
        if self.errors:
            raise ModelValidationError(self.errors)
        try:
            self.instance.clean_fields()
        except ValidationError as e:
            raise ModelValidationError(e.message_dict)
        self.instance.save()

        # Serialize related fields
        for key, val in self.data.items():
            if key not in self.related_fields:
                continue
            self._deserialize_related(key, val)

        # Serialize m2m fields
        for key, val in self.data.items():
            if key not in self.m2m_fields:
                continue
            self._deserialize_related(key, val, save_instance=True)
        return self.instance

    def _deserialize_field(self, key, val):
        if hasattr(self, key):
            serializer = self._get_related_serializer(key)
            value = serializer(val).save()
            setattr(self.instance, key, value)
            value.save()
            return

        field = self.opts.model._meta.get_field(key)
        field_type = field.__class__.__name__
        deserializer = get_deserializer(field_type)
        if deserializer:
            deserializer(self.instance, key, val)
        else:
            setattr(self.instance, key, val)

    def _deserialize_related(self, key, val, save_instance=False):
        serializer = self._get_related_serializer(key)
        if isinstance(val, list):
            for v in val:
                related_instance = serializer(v).deserialize()
                if save_instance:
                    related_instance.save()
                getattr(self.instance, key).add(related_instance)
        else:
            related_instance = serializer(val).deserialize()
            setattr(self.instance, key, related_instance)

    def validate_field(self, field, value, data):
        validation_name = 'validate_{}'.format(field)
        if hasattr(self, validation_name):
            validator = getattr(self, validation_name)
            validator(value)
        return None


    def _get_related_serializer(self, key):
        serializer = getattr(self, key, None)
        if isinstance(serializer, str):
            return get_serializer(serializer, self.__class__)
        return serializer

    def _get_custom_field_serializers(self):
        """
        Get all custom serializer functions.
        If this function has a serializer attached to it, include that
        """
        functions = [(
            getattr(self, f),
            f.replace('serialize_', '')
        ) for f in dir(self) if f.startswith('serialize_')]
        return functions

    def get_object_map_data(self):
        return {
            'id': getattr(self.instance, self.opts.id_field),
            '_type': self.opts.model._meta.model_name
        }

    def serialize(self, fields=None, ignore_serializers=None):
        if not fields:
            fields = self.opts.publish_fields
        if not self.instance:
            return None

        data = self.get_object_map_data()

        # Set all the ids for related models
        # so the datamapper can find the connection
        data.update(get_id_mappings(self))

        # Set the id value for related models, for the data mapper
        # if ignore_serializers:
        #     for ser in ignore_serializers:
        #         via = '{}'.format(get_serializer_relationship_field(ser, self))
        #         if hasattr(self.instance, via):
        #             data[via] = getattr(self.instance, via)

        # Serialize the fields
        for field in fields:
            data[field] = self._serialize_value(field, ignore_serializers)

        custom_serializer_functions = self._get_custom_field_serializers()
        for custom_function, name in custom_serializer_functions:
            serializer = getattr(self, name, None)
            if serializer:
                serializer = get_serializer(serializer, self)
                data[name] = custom_function(self.instance, serializer)
            else:
                data[name] = custom_function(self.instance)

        return data

    def _serialize_value(self, attr_name, ignore_serializers=None):
        obj_serializer = self._get_related_serializer(attr_name)
        # To prevent infinite recursion, allow serializers to be ignored
        if ignore_serializers and obj_serializer in ignore_serializers:
            return None

        val = get_property(self.instance, attr_name)

        # If we have one or more related models
        if obj_serializer and hasattr(val, 'all'):
            return [obj_serializer(instance=o).serialize(ignore_serializers=[self.__class__]) for o in val.all()]
        elif obj_serializer:
            return obj_serializer(instance=val).serialize(ignore_serializers=[self.__class__])
        elif hasattr(self.opts.model, attr_name):
            # Check if the field is a relation of any kind
            field_type = getattr(self.opts.model, attr_name)
            # Reverse FK
            if isinstance(field_type, ReverseSingleRelatedObjectDescriptor):
                val = get_property(self.instance, attr_name).pk
            # FK
            elif isinstance(field_type, ForeignRelatedObjectsDescriptor):
                val = list(get_property(self.instance, attr_name).all().values_list('pk', flat=True))
            elif isinstance(field_type, ReverseManyRelatedObjectsDescriptor):
                val = list(get_property(self.instance, attr_name).all().values_list('pk', flat=True))
            elif isinstance(field_type, ManyRelatedObjectsDescriptor):
                val = list(get_property(self.instance, attr_name).all().values_list('pk', flat=True))

        # Serialize the field
        return serialize_field(val)

    @classmethod
    def get_object_map(cls, include_serializers=None, ignore_serializers=None):
        return get_object_map(cls, ignore_serializers)

    @classmethod
    def get_base_channel(cls):
        if hasattr(cls.Meta, 'base_channel'):
            return '{}|'.format(getattr(cls.Meta, 'base_channel'))
        return '{}|'.format(get_model(cls.Meta.model)._meta.model_name)

    @classmethod
    def get_related_serializers(cls):
        possible_serializers = [k for k in cls.__dict__.keys() if not k.startswith('_') and not k == 'Meta']
        serializers = []
        for possible_serializer in possible_serializers:
            val = getattr(cls, possible_serializer)
            if isinstance(val, str):
                val = get_serializer(val, cls)
            if hasattr(val, 'serialize'):
                serializers.append((val, possible_serializer))
        return serializers



from ..route_handler import BaseModelPublisherRouter, SUCCESS, ERROR
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        publish_fields = ('id')
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelPublisherRouter):
    model = TwoFieldModel
    serializer_class = Serializer

    def get_object(self, **kwargs):
        return self.model.objects.get(**kwargs)


class TestBaseModelPublisherRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)
        self.obj = self.router.model.objects.create(text='text', number=5)

    def test_deleted(self):
        data = {'id': self.obj.pk}
        self.router.subscribe(**{'channel': 'client-channel'})
        self.router.delete(**data)
        actual = self.connection.get_last_published()
        expected = {'action': 'deleted', 'channel': 'twofieldmodel|', 'data': {'_type': 'twofieldmodel', 'id': 1}}
        self.assertDictEqual(actual, expected)



from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor, SingleRelatedObjectDescriptor, \
    ForeignRelatedObjectsDescriptor, ManyRelatedObjectsDescriptor, ReverseManyRelatedObjectsDescriptor


def _construct_graph(parent_type, child_type, via, is_collection, property_name):
    return {
        'parent_type': parent_type,
        'child_type': child_type,
        'via': via,
        'prop_name': property_name,
        'is_collection': is_collection,
    }


def _serializer_is_ignored(serializer, related_serializer, ignore_serializer_pairs):
    if not ignore_serializer_pairs:
        return False
    if (serializer, related_serializer) in ignore_serializer_pairs:
        return True
    return False


def get_object_map(serializer, ignore_serializer_pairs=None):
    """
    Create an object map from the serializer and it's related serializers.

    For each map created, ignore the pair of serializers that are already mapped
    """
    graph = []
    serializer_instance = serializer()
    if ignore_serializer_pairs is None:
        ignore_serializer_pairs = []

    serializers = serializer.get_related_serializers()

    for related_serializer, field_name in serializers:
        if _serializer_is_ignored(serializer, related_serializer, ignore_serializer_pairs):
            continue

        field_type = getattr(serializer_instance.opts.model, field_name)
        is_fk = isinstance(field_type, ReverseSingleRelatedObjectDescriptor)
        is_o2o = isinstance(field_type, SingleRelatedObjectDescriptor)
        is_reverse_fk = isinstance(field_type, ForeignRelatedObjectsDescriptor)
        is_m2m = isinstance(field_type, ManyRelatedObjectsDescriptor)
        is_reverse_m2m = isinstance(field_type, ReverseManyRelatedObjectsDescriptor)

        if is_fk:
            model = field_type.field.related.parent_model
            is_collection = False
            attname = field_type.field.related.var_name

        if is_o2o:
            model = field_type.related.model
            is_collection = False
            attname = field_type.related.field.name

        if is_reverse_fk:
            model = field_type.related.model
            is_collection = True
            attname = field_type.related.field.name

        if is_m2m:
            model = field_type.related.model
            is_collection = True
            attname = field_type.related.field.name

        if is_reverse_m2m:
            model = field_type.field.related.parent_model
            is_collection = True
            attname = field_type.field.related.var_name

        # if hasattr(field, 'related'):
        #     model = field.related.model
        #     attname = field.related.field.name
        #     is_collection = True
        # else:
        #     model = field.field.related.parent_model
        #     attname = field.field.related.var_name
        #     is_collection = False
        graph.append(
            _construct_graph(
                serializer_instance.opts.model._meta.model_name,
                model._meta.model_name,
                attname,
                is_collection,
                field_name
            )
        )
        ignore_serializer_pairs.append((serializer, related_serializer))
        graph += get_object_map(related_serializer, ignore_serializer_pairs)
    return graph



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TextModel


class Serializer(ModelSerializer):
    class Meta:
        model = TextModel


class Router(BaseModelRouter):
    model = TextModel
    serializer_class = Serializer

    def get_object(self, **kwargs):
        return self.model.objects.first()


class TestBaseModelRouter(DragonTestCase):
    def test_get_object(self):
        TextModel.objects.create(text='This is a text model')

        router = Router(self.connection)
        with self.assertNumQueries(1):
            for i in range(2):
                router.get_single()



try:
    from urllib.parse import quote_plus, unquote_plus
except ImportError:
    from urllib import quote_plus, unquote_plus


def term_match_check(term, val, option):
    decoded_val = unquote_plus(val)
    comparer = term_comparison_factory(option)
    return comparer(decoded_val, term)


def standard_compare(term, val):
    if val is None:
        return term is val
    term = type(val)(term)
    return term == (val)


def contains_compare(term, val):
    return term in val


def in_compare(term, val):
    if not val or not term:
        return False
    return val in [type(val)(t) for t in term]


def lt_compare(term, val):
    term = type(val)(term)
    return val < term


def lte_compare(term, val):
    term = type(val)(term)
    return val <= term


def gt_compare(term, val):
    term = type(val)(term)
    return val > term


def gte_compare(term, val):
    term = type(val)(term)
    return val >= term


filter_options = {
    'contains': contains_compare,
    'lt': lt_compare,
    'lte': lte_compare,
    'gt': gt_compare,
    'gte': gte_compare,
    'eq': standard_compare,
    'in': in_compare,
}


def term_comparison_factory(option):
    if option is None:
        return standard_compare
    return filter_options.get(option)



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_get_list(self):
        with self.assertRaises(NotImplementedError):
            self.router.get_list()

    def test_get_single(self):
        with self.assertRaises(NotImplementedError):
            self.router.get_single()

    def test_create(self):
        with self.assertRaises(NotImplementedError):
            self.router.create()

    def test_update(self):
        with self.assertRaises(NotImplementedError):
            self.router.update()

    def test_delete(self):
        with self.assertRaises(NotImplementedError):
            self.router.delete()



from .redis_pubsub_provider import RedisPubSubProvider
from .mock_pubsub_provider import MockPubSubProvider
import sys

_subscriber = None

def get_subscription_provider():
    global _subscriber
    if not _subscriber:
        if 'test' in sys.argv:
            _subscriber = MockPubSubProvider()
        else:
            _subscriber = RedisPubSubProvider()
    return _subscriber



from ..route_handler import BaseRouter, BaseModelRouter
from .. import route_handler
from .dragon_test_case import DragonTestCase


class Router(BaseRouter):
    route_name = 'test-router'


class ModelRouterGetObject(BaseModelRouter):
    route_name = 'model-router'

    def get_object(self, **kwargs):
        pass


class ModelRouterGetQuerySet(BaseModelRouter):
    route_name = 'model-router'

    def get_query_set(self, **kwargs):
        pass


class TestRouteHandler(DragonTestCase):
    def test_register_router(self):
        route_handler.register(Router)
        self.assertIsNotNone(route_handler.get_route_handler(Router.get_name()))

    def test_register_router_twice(self):
        route_handler.register(Router)
        route_handler.register(Router)
        self.assertEqual(len(route_handler.registered_handlers), 1)

    def test_register_model_router_with_missing_get_object(self):
        with self.assertRaises(Exception):
            route_handler.register(ModelRouterGetObject)

    def test_register_model_router_with_missing_get_query_set(self):
        with self.assertRaises(Exception):
            route_handler.register(ModelRouterGetQuerySet)



from ..cache.cache_factory import get_cache_store
from ..serializers.model_serializer import ModelSerializer


class CacheModelSerializer(ModelSerializer):
    def __init__(self, *args, **kwargs):
        super(CacheModelSerializer, self).__init__(*args, **kwargs)
        self._cache = get_cache_store()

    def get_cache_key(self):
        return 'cache_{}{}'.format(self.get_base_channel(), self.instance.pk)

    def remove_from_cache(self):
        key = self.get_cache_key()
        self._cache.remove(key)

    def serialize(self, ignore_serializers=None):
        key = self.get_cache_key()
        if self.instance:
            if self._cache.has_data(key):
                return self._cache.get_data(key)
        data = super().serialize(ignore_serializers=ignore_serializers)
        if data:
            self._cache.store_data(data, key, 30)
        return data



def discover_routes():
    from swampdragon import route_handler
    """
    Discover, borrowed from django admin.
    Returns urls for each route handler
    """
    from django.conf import settings
    from django.utils.importlib import import_module
    imported_routers = []
    urls = []
    for app in settings.INSTALLED_APPS:
        try:
            target_mod = '%s.routers' % app
            if target_mod not in imported_routers:
                import_module(target_mod)
            imported_routers.append(target_mod)
        except ImportError:
            pass
    routes = route_handler.registered_handlers
    for route in routes:
        urls.append(('/' + route + '/$', routes[route]))
    return urls


def load_field_deserializers():
    from django.conf import settings
    from django.utils.importlib import import_module
    imported_deserializers = []
    for app in settings.INSTALLED_APPS:
        try:
            target_mod = '%s.field_deserializers' % app
            if target_mod not in imported_deserializers:
                import_module(target_mod)
            imported_deserializers.append(target_mod)
        except ImportError:
            pass



from django.db.models.loading import get_model as get_django_model


def get_property(obj, field):
    field = field.replace('__', '.')
    if '.' in field:
        attr_chain = field.split('.')
        attr = obj
        for elem in attr_chain:
            attr = getattr(attr, elem, None)
            if attr is None:
                return None
        return attr
    try:
        return getattr(obj, field, None)
    except:
        return None


def string_to_list(val):
    return val.replace('[', '').replace(']', '').split(',')


def get_model(model):
    if isinstance(model, str):
        return get_django_model(*model.split('.', 1))
    return model


def get_m2m_model_from_field_name(instance, field_name):
    try:
        return getattr(instance, field_name).model
    except:
        return None


def get_field_from_m2m_model_by_model(instance, model):
    """
    Provided an instance and a model, find the
    field name on the instance, where it has an m2m relationship
    to the model
    """
    for f, ignore in instance._meta.get_all_related_m2m_objects_with_model():
        if f.model == model:
            return f.get_accessor_name()

    for f in instance._meta.get_all_field_names():
        try:
            field = instance._meta.get_field_by_name(f)[0]
            if hasattr(field, 'get_accessor_name'):
                name = field.get_accessor_name()
            else:
                name = field.name
            if get_m2m_model_from_field_name(instance, name):
                return name
        except:
            continue
    return None


from ..pubsub_providers.base_provider import BaseProvider
from . import mock_publisher
from swampdragon.pubsub_providers.publisher_factory import get_publisher

_channels = []
publisher = get_publisher()


class MockPubSubProvider(BaseProvider):
    def __init__(self):
        self.publisher = publisher

    def publish(self, channel, data):
        connections = publisher.subscribers.get(channel, [])
        for c in connections:
            c.publish(data)

    def subscribe(self, channels, connection):
        connection.channels += channels
        for c in channels:
            if not c in publisher.subscribers.keys():
                publisher.subscribers[c] = []
            publisher.subscribers[c].append(connection)

    def unsubscribe(self, channels, connection):
        for c in channels:
            if c in connection.channels:
                connection.channels.remove(c)
            if c in publisher.subscribers.keys():
                if connection in publisher.subscribers[c]:
                    publisher.subscribers[c].remove(connection)
                if not publisher.subscribers[c]:
                    del publisher.subscribers[c]
        connection.channels = []

    def close(self, connection):
        self.unsubscribe(connection.channels, connection)



import sys
from swampdragon.pubsub_providers import redis_publisher, mock_publisher

_publisher = None


def get_publisher():
    global _publisher
    if not _publisher:
        if 'test' in sys.argv:
            _publisher = mock_publisher.MockPublisher()
        else:
            _publisher = redis_publisher
    return _publisher



from sockjs.tornado import SockJSConnection
from ..pubsub_providers.subscriber_factory import get_subscription_provider
from .. import route_handler
import json


class ConnectionMixin(object):
    def to_json(self, data):
        """
        If the message is a dict, return it.
        If it's a string try to decode it into json,
        otherwise assume it's not a dictionary but a simple text message
        """
        if isinstance(data, dict):
            return data
        try:
            return json.loads(data)
        except:
            return {'message': data}


class SubscriberConnection(ConnectionMixin, SockJSConnection):
    channels = []
    pub_sub = None

    def __init__(self, session):
        super(SubscriberConnection, self).__init__(session)
        self.pub_sub = get_subscription_provider()

    def on_close(self):
        self.pub_sub.close(self)

    def on_message(self, data):
        try:
            data = self.to_json(data)
            handler = route_handler.get_route_handler(data['route'])
            handler(self).handle(data)
        except Exception as e:
            self.abort_connection()
            raise e

    def abort_connection(self):
        self.close()


class DjangoSubscriberConnection(SubscriberConnection):
    pass



"""
Django settings for selfpub_example project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '^dm!dv5s=h*)e^e(q@q0i7!fdhum2f4yedhr5*4wmq0+$=!gz2'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'swampdragon',
    'swampdragon.tests',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'urls'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'dev.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = '/media/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static_root')
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, 'static')
]

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATE_DIRS = [
    os.path.join(BASE_DIR, 'templates')
]

SOCKJS_CLASSES = (
    ('selfpub_example.sockserver.DataConnection', '/data'),
)



import json


subscribers = {}


class MockPublisher(object):
    def __init__(self):
        self.subscribers = subscribers

    def publish(self, channel, message):
        subs = subscribers.get(channel)
        if not subs:
            return
        for subscriber in subs:
            if isinstance(message, str):
                message = json.dumps(message)
            subscriber.published_data.append(message)

    def _get_channels_from_subscriptions(self, base_channel):
        channels = [key for key in self.subscribers.keys() if key.startswith(base_channel)]
        return channels

    def get_channels(self, base_channel):
        return self._get_channels_from_subscriptions(base_channel)



from dateutil.parser import parse


deserializer_map = {}


def register_field_deserializer(field_name, deserializer):
    deserializer_map[field_name] = deserializer


class BaseFieldDeserializer(object):
    def __call__(self, *args, **kwargs):
        raise NotImplemented()


class DateTimeDeserializer(BaseFieldDeserializer):
    def __call__(self, model_instance, key, val):
        date_val = parse(val)
        setattr(model_instance, key, date_val)


class DateDeserializer(DateTimeDeserializer):
    pass


register_field_deserializer('DateTimeField', DateTimeDeserializer)
register_field_deserializer('DateField', DateDeserializer)


def get_deserializer(name):
    if name in deserializer_map:
        return deserializer_map[name]()
    return None



from ..route_handler import BaseRouter, UnexpectedVerbException
from .dragon_test_case import DragonTestCase
from swampdragon.permissions import RoutePermission


class FooRouter(BaseRouter):
    valid_verbs = ('call_foo')
    route_name = 'foo'
    router_called = False

    def call_foo(self, **kwargs):
        self.router_called = True


class Permission(RoutePermission):
    def test_permission(self, handler, verb, **kwargs):
        return kwargs['permission_pass']

    def permission_failed(self, handler):
        pass


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)

    def test_handle(self):
        data = {'verb': 'call_foo'}
        self.router.handle(data)
        self.assertTrue(self.router.router_called)

    def test_invalid_verb(self):
        data = {'verb': 'invalid_verb'}
        with self.assertRaises(UnexpectedVerbException):
            self.router.handle(data)

    def test_page_context(self):
        data = {'verb': 'call_foo', 'args': {'_page': 12}}
        self.router.handle(data)
        self.assertIn('page', self.router.context)

    def test_permission_fail(self):
        FooRouter.permission_classes = (Permission(), )
        data = {'verb': 'call_foo', 'args': {'permission_pass': False}}
        self.router.handle(data)
        self.assertFalse(self.router.router_called)

    def test_permission_pass(self):
        FooRouter.permission_classes = (Permission(), )
        data = {'verb': 'call_foo', 'args': {'permission_pass': True}}
        self.router.handle(data)
        self.assertTrue(self.router.router_called)



import redis
import json
import tornadoredis.pubsub
import tornadoredis
from .base_provider import BaseProvider


# redis_client = redis.StrictRedis()
# subscriber = tornadoredis.pubsub.SockJSSubscriber(tornadoredis.Client())


class RedisPubSubProvider(BaseProvider):
    def __init__(self):
        self._client = redis.StrictRedis()
        self._subscriber = tornadoredis.pubsub.SockJSSubscriber(tornadoredis.Client())

    def close(self, broadcaster):
        for channel in broadcaster.channels:
            self._subscriber.unsubscribe(channel, broadcaster)

    def get_channel(self, base_channel, **channel_filter):
        return self._construct_channel(base_channel, **channel_filter)

    def subscribe(self, channels, broadcaster):
        self._subscriber.subscribe(channels, broadcaster)
        for channel in channels:
            if channel not in broadcaster.channels:
                broadcaster.channels.append(channel)

    def unsubscribe(self, channels, broadcaster):
        for channel in channels:
            self._subscriber.unsubscribe(channel, broadcaster)

    def publish(self, channel, data):
        if isinstance(data, dict):
            data = json.dumps(data)
        broadcasters = list(self._subscriber.subscribers[channel].keys())
        if broadcasters:
            for bc in broadcasters:
                if not bc.session.is_closed:
                    bc.broadcast(broadcasters, data)
                    break



from ..route_handler import BaseModelRouter
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        model = TwoFieldModel


class TestBaseModelRouter(DragonTestCase):
    def test_get_changes(self):
        router = BaseModelRouter(self.connection)
        router.serializer = Serializer(instance=TwoFieldModel())
        current_state = {'changed': 'a', 'nochange': 123, 'id': 1}
        past_state = {'changed': 'b', 'nochange': 123, 'id': 1}
        changed_fields = router._get_changed_fields(current_state, past_state)
        expected = ['changed']
        self.assertListEqual(expected, changed_fields)



_imported_modules_ = {}
_imported_serializers_ = {}


def add_module_to_imported_modules(src, mod):
    _imported_modules_[src] = mod


def get_module(src):
    if src in _imported_modules_.keys():
        return _imported_modules_[src]
    return None


def get_serializer(ser, f):
    if ser in _imported_serializers_:
        return _imported_serializers_[ser]

    root = f.__module__.split('.')[0]
    package_name = ser.rsplit('.', 1)[0]
    if '.' in ser:
        serializer_class_name = ser.rsplit('.', 1)[1]
        src = '.'.join([root, package_name, 'serializers'])
    else:
        serializer_class_name = ser
        src = f.__module__

    try:
        mod = get_module(src)
        if not mod:
            mod = __import__(src, fromlist=[serializer_class_name])
            add_module_to_imported_modules(src, mod)
        klass = getattr(mod, serializer_class_name)
        _imported_serializers_[ser] = klass
        return klass
    except:
        pass



from django.conf import settings
from .session_store import BaseSessionStore
import redis


class RedisSessionStore(BaseSessionStore):
    def __init__(self):
        self.client = redis.StrictRedis()

    def save(self, key, val):
        self.client.set(key, val)
        self.client.expire(key, getattr(settings, 'SESSION_EXPIRATION_TIME', 30) * 60)

    def get(self, key):
        return self.client.get(key)

    def refresh_key_timeout(self, key):
        self.client.expire(key, getattr(settings, 'SESSION_EXPIRATION_TIME', 30) * 60)



from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import ParentModel, ChildModel


class ParentSerializer(ModelSerializer):
    childmodel_set = 'ChildSerializer'

    class Meta:
        model = ParentModel
        publish_fields = ('text', 'childmodel_set')


class ChildSerializer(ModelSerializer):
    parent = ParentSerializer

    class Meta:
        model = ChildModel
        publish_fields = ('number', 'parent')


class ChildModelSerializer(ModelSerializer):
    class Meta:
        model = ChildModel
        publish_fields = ('number', 'parent',)


class ParentModelSerializer(ModelSerializer):
    class Meta:
        model = ParentModel
        publish_fields = ('name', 'childmodel_set')


class TestModelSerializer(DragonTestCase):
    def test_serialize_fk(self):
        parent = ParentModel.objects.create(name='hello world')
        child = ChildModel.objects.create(number=5, parent=parent)
        result = ChildSerializer(instance=child).serialize()
        self.assertIsNotNone(result['parent'])

    def test_reverse_fk(self):
        parent = ParentModel.objects.create(name='hello world')
        child = ChildModel.objects.create(number=5, parent=parent)
        result = ParentSerializer(instance=parent).serialize()
        self.assertIsNotNone(result['childmodel_set'])

    def test_serialize_ignore_related(self):
        parent = ParentModel.objects.create(name='hello world')
        child = ChildModel.objects.create(number=5, parent=parent)
        result = ChildSerializer(instance=child).serialize(ignore_serializers=[ParentSerializer])
        self.assertIsNone(result['parent'])


    def test_serialize_fk_without_serializer(self):
        """
        If a related model is included but no serializer specified, then include the
        ids instead
        """
        parent = ParentModel.objects.create(name='perry')
        child = ChildModel.objects.create(number=2, parent=parent)
        ser = ChildModelSerializer(instance=child)
        data = ser.serialize()
        self.assertEqual(data['parent'], parent.pk)

    def test_serialize_reverse_fk_without_serializer(self):
        """
        If a related model is included but no serializer specified, then include the
        ids instead
        """
        parent = ParentModel.objects.create(name='perry')
        child_a = ChildModel.objects.create(number=2, parent=parent)
        child_b = ChildModel.objects.create(number=5, parent=parent)
        ser = ParentModelSerializer(instance=parent)
        data = ser.serialize()
        self.assertListEqual(list(data['childmodel_set']), [child_a.pk, child_b.pk])



from ..connections.sockjs_connection import SubscriberConnection
from ..route_handler import BaseRouter, UnexpectedVerbException
from .dragon_test_case import DragonTestCase
from .. import route_handler


class TestSession(object):
    def __init__(self, is_open=True):
        self.is_closed = is_open == False
        self.messages = []

    def send_message(self, message, binary=False):
        self.messages.append(message)

    def close(self):
        self.is_closed = True


class TestRouter(BaseRouter):
    route_name = 'test-router'
    valid_verbs = ['say_hello']

    def say_hello(self, **kwargs):
        self.connection.hello_said = True


class TestSubscriberConnection(DragonTestCase):
    def setUp(self):
        self.session = TestSession()
        self.connection = SubscriberConnection(self.session)

    def test_on_close(self):
        """
        Closing a connection should automatically unsubscribe
        from all channels
        """
        self.connection.on_open(None)
        self.connection.pub_sub.subscribe(['test-channel'], self.connection)
        self.assertIn(self.connection, self.connection.pub_sub.publisher.subscribers['test-channel'])
        self.connection.on_close()
        self.assertNotIn('test-channel', self.connection.pub_sub.publisher.subscribers)

    def test_on_message(self):
        route_handler.register(TestRouter)
        data = {'verb': 'say_hello', 'route': TestRouter.get_name()}
        self.connection.on_message(data)
        self.assertTrue(self.connection.hello_said)

    def test_on_message_invalid_verb(self):
        route_handler.register(TestRouter)
        data = {'verb': 'invalid_verb', 'route': TestRouter.get_name()}
        with self.assertRaises(UnexpectedVerbException):
            self.connection.on_message(data)
        self.assertTrue(self.connection.is_closed)

    def test_string_to_json(self):
        data = '{"key": "val"}'
        self.assertDictEqual({'key': 'val'}, self.connection.to_json(data))

    def test_to_json_plain_text(self):
        expected = {'message': 'hello'}
        self.assertDictEqual(expected, self.connection.to_json('hello'))

    def test_dict_to_json(self):
        """
        If the message received is a dictionary simply return it
        """
        expected = {'key': 'value'}
        actual = self.connection.to_json(expected)
        self.assertEqual(expected, actual)



from ..route_handler import BaseRouter
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'


class NamelessRrouter(BaseRouter):
    pass


class TestBaseRouter(DragonTestCase):
    def test_get_name(self):
        self.assertEqual(FooRouter.get_name(), FooRouter.route_name)

    def test_name_missing(self):
        with self.assertRaises(Exception):
            NamelessRrouter.get_name()



from ..route_handler import BaseRouter, ERROR, SUCCESS, LOGIN_REQUIRED
from .dragon_test_case import DragonTestCase


class FooRouter(BaseRouter):
    route_name = 'foo'


class BarRouter(BaseRouter):
    channels = ['channel_a', 'channel_b']

    def get_subscription_channels(self, **kwargs):
        return self.channels


class TestBaseRouter(DragonTestCase):
    def setUp(self):
        self.foo_router = FooRouter(self.connection)
        self.bar_router = BarRouter(self.connection)

    def test_get_subscription_channel(self):
        with self.assertRaises(NotImplementedError):
            self.foo_router.get_subscription_channels()

        channel = self.bar_router.get_subscription_channels()
        self.assertListEqual(channel, BarRouter.channels)



from tornado.web import RequestHandler
from swampdragon.default_settings import SwampDragonSettings


class SettingsHandler(RequestHandler):

    def set_default_headers(self):
        self.set_header("Content-Type", "text/javascript")

    def get(self, *args, **kwargs):
        data = '''window.swampdragon_settings = {settings};
window.swamp_dargon_host = "{protocol}://{host}";
        '''.format(**{
            'settings': SwampDragonSettings().to_dict(),
            'host': self.request.headers['Host'],
            'protocol': self.request.protocol,
        })
        self.write(data)



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="SwampDragon",
    version="0.3.5.1",
    author="Jonas Hagstedt",
    author_email="hagstedt@gmail.com",
    description=("SwampDragon is a powerful platform making it easy to build real time web applications, combining the power of Django and Tornado"),
    license="BSD",
    keywords="SwampDragon, realtime, sockjs, django, tornado, framework",
    url="http://swampdragon.net",
    packages=find_packages(),
    long_description=read('README.md'),
    include_package_data=True,
    entry_points={'console_scripts': ['dragon-admin = swampdragon.core:run',]},
    install_requires=[
        "Django >= 1.4",
        "Tornado",
        "sockjs-tornado",
        "tornado-redis",
        "redis",
        "python-dateutil"
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
)



from collections import namedtuple
from django.db.models.fields.related import ForeignKey, ReverseSingleRelatedObjectDescriptor, \
    ManyRelatedObjectsDescriptor, ReverseManyRelatedObjectsDescriptor, ForeignRelatedObjectsDescriptor, \
    SingleRelatedObjectDescriptor
from django.db.models.related import RelatedObject
from django.db.models.fields.related import ManyToManyField


class FieldType(namedtuple('FieldType', 'field, model, fk, m2m')):
    '''
    Determine if a field is an m2m, reverse m2m, fk or reverse fk
    '''
    @property
    def is_m2m(self):
        return self.fk is False and self.m2m is True and isinstance(self.field, RelatedObject)

    @property
    def is_reverse_m2m(self):
        return self.fk is True and self.m2m is True and isinstance(self.field, ManyToManyField)

    @property
    def is_fk(self):
        return self.fk is True and self.m2m is False and isinstance(self.field, ForeignKey)

    @property
    def is_reverse_fk(self):
        return self.fk is False and self.m2m is False and isinstance(self.field, RelatedObject)


def get_serializer_relationship_field(serializer, related_serializer):
    if isinstance(serializer, type):
        model = serializer().opts.model
    else:
        model = serializer.opts.model
    if isinstance(related_serializer, type):
        related_model = related_serializer().opts.model
    else:
        related_model = related_serializer.opts.model

    for field_name in related_model._meta.get_all_field_names():
        field_type = FieldType(*related_model._meta.get_field_by_name(field_name))
        field = field_type.field

        # Foreign key
        if field_type.is_fk and field.rel.to is model:
            return field.verbose_name

        # Reverse foreign key
        if field_type.is_reverse_fk and field.model is model:
            return field.var_name

        # M2m fields
        if field_type.is_m2m and field.model is model:
            return field.var_name

        # Reverse m2m field
        if field_type.is_reverse_m2m and field.rel.to is model:
            return field.attname


def get_id_mappings(serializer):
    if not serializer.instance:
        return {}

    data = {}
    for field_name in serializer.opts.publish_fields:
        if not hasattr(serializer, field_name):
            continue

        serializable_field = serializer._get_related_serializer(field_name)
        if not hasattr(serializable_field, 'serialize'):
            continue

        field_type = getattr(serializer.opts.model, field_name)
        is_fk = isinstance(field_type, ReverseSingleRelatedObjectDescriptor)
        is_o2o = isinstance(field_type, SingleRelatedObjectDescriptor)
        is_reverse_fk = isinstance(field_type, ForeignRelatedObjectsDescriptor)
        is_m2m = isinstance(field_type, ManyRelatedObjectsDescriptor)
        is_reverse_m2m = isinstance(field_type, ReverseManyRelatedObjectsDescriptor)

        try:
            val = getattr(serializer.instance, field_name)
        except:
            continue

        if not val:
            continue

        if is_fk or is_o2o:
            data['{}'.format(field_name)] = val.pk
            continue

        if is_reverse_fk or is_m2m or is_reverse_m2m:
            data['{}'.format(field_name)] = list(val.all().values_list('pk', flat=True))
            continue

    return data



from ..pubsub_providers.publisher_factory import get_publisher


def publish_data(channel, data):
    publisher = get_publisher()
    pub_data = {'data': data, 'channel': channel}
    publisher.publish(channel, pub_data)



from ..model_tools import string_to_list, get_property
from .channel_filters import filter_options, in_compare, term_match_check


try:
    from urllib.parse import quote_plus, unquote_plus
except ImportError:
    from urllib import quote_plus, unquote_plus


def make_safe(val):
    """
    Make strings in filters save.
    i.e 'foo bar' becomes 'foo+bar'
    """
    if not isinstance(val, str):
        return val
    return quote_plus(val)


def remove_channel_filter(channel):
    """
    Remove filters from channel strings
    i.e foo_contains becomes foo
    """
    if '__' not in channel:
        return channel
    chan, channel_filter = channel.rsplit('__', 1)
    if filter_options.get(channel_filter):
        return chan
    return channel


def get_channel_filter(channel):
    if '__' not in channel:
        return filter_options['eq']
    chan, channel_filter_name = channel.rsplit('__', 1)
    channel_filter = filter_options.get(channel_filter_name)
    if not channel_filter:
        return filter_options['eq']
    return channel_filter


def get_property_and_value_from_channel(channel):
    """
    Get a list of tuples with properties and channels.
    i.e foo|bar__name__contains:baz returns a list: [('bar__name__contains', 'baz')]
    """
    filters = filter(None, str(channel).split('|')[1:])
    if not filters:
        return None
    properties = []
    for channel_filter, val in [tuple(f.split(':', 1)) for f in filters]:
        filter_option = filter_options.get(channel_filter.split('__')[-1])
        if filter_option == in_compare:
            val = string_to_list(val)
        properties.append((channel_filter, val))
    return properties


def channel_match_check(channel, data):
    terms = filter(None, channel.split('|')[1:])
    option = None
    for term in terms:
        key, val = term.split(':')
        if '__' in key and key.split('__')[-1] in filter_options.keys():
            option = key.rsplit('__', 1)[-1]
        if key not in data:
            return False
        if not term_match_check(data[key], val, option):
            return False
    return True


def properties_match_channel_by_object(obj, channel_properties):
    result = True
    for prop, val in channel_properties:
        if not has_val(obj, prop, val) and not has_related_value(obj, prop, val):
            return False
    return result


def properties_match_channel_by_dict(dict, channel_properties):
    result = True
    for prop, val in channel_properties:
        if prop not in dict:
            return False
        val_type = type(val)
        if not val_type(dict[prop]) == val:
            return False
    return result


def get_value(obj, prop):
    data = {}
    val = get_property(obj, prop)
    if val:
        data[prop] = val
    return data


def has_val(obj, prop, val):
    obj_val = get_property(obj, remove_channel_filter(prop))
    if not obj_val:
        return False
    channel_filter = get_channel_filter(prop)
    return channel_filter(val, obj_val)


def has_related_value(obj, field, channel_val):
    if '__' not in field:
        filter_by_val = channel_val
        property_name = field
    else:
        property_name, filter_by_val = field.split('__', 1)
    attr = getattr(obj, property_name)
    if hasattr(attr, 'all'):
        return getattr(obj, property_name).filter(**{filter_by_val: channel_val}).exists()
    else:
        filter_query = {'pk': obj.pk}
        filter_query[field] = channel_val
        return obj.__class__.objects.filter(**filter_query).exists()



from django.core.management import call_command
import sys
from os.path import abspath, dirname, join
from os import path
import os


def _add_swampdragon_to_installed_apps(settings):
    installed_apps_start = -1
    installed_apps_end = -1
    for i in range(0, len(settings)):
        if settings[i].decode().startswith('INSTALLED_APPS'):
            installed_apps_start = i
            break
    if installed_apps_start == -1:
        raise Exception('no installed apps found')

    for i in range(installed_apps_start, len(settings)):
        if ')' in settings[i].decode():
            installed_apps_end = i
            break
    swampdragon = '    \'swampdragon\',\n'.encode('utf-8')
    settings.insert(installed_apps_end, swampdragon)
    return settings


def run():
    args = sys.argv
    if len(args) is 1:
        print('no command selected')
        return
    if 'startproject' in args:
        if len(args) < 3:
            print('supply a project name')
            return
        start_project(args[2])


def add_server_py(project_name):
    root = dirname(dirname(abspath(__file__)))
    template_dir = path.join(root, 'app_templates')
    serverpy_template = path.join(template_dir, 'server.py')
    with open(serverpy_template, 'rb') as serverpy_file:
        serverpy_lines = serverpy_file.readlines()
        serverpy_text = []
        for line in serverpy_lines:
            line = line.replace(b'<project>', project_name.encode('utf-8'))
            serverpy_text.append(line)

    current_dir = os.getcwd()
    serverpy_dest = path.join(path.join(path.join(current_dir, project_name)), 'server.py')

    with open(serverpy_dest, 'wb') as project_serverpy_file:
        project_serverpy_file.writelines(serverpy_text)


def start_project(project_name):
    call_command('startproject', project_name)
    root = dirname(dirname(abspath(__file__)))
    template_dir = path.join(root, 'app_templates')
    settings_template = path.join(template_dir, 'sd_settings.py')

    current_dir = os.getcwd()
    project_settings_file = path.join(path.join(path.join(current_dir, project_name), project_name), 'settings.py')
    with open(settings_template, 'rb') as template_file:
        socket_settings = template_file.readlines()

    with open(project_settings_file, 'rb') as settings_file:
        settings = settings_file.readlines()
    settings = _add_swampdragon_to_installed_apps(settings)
    settings += socket_settings

    with open(project_settings_file, 'wb') as settings_file:
        settings_file.writelines(settings)

    add_server_py(project_name)



from ..route_handler import BaseModelPublisherRouter, SUCCESS, ERROR
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from swampdragon.tests.models import TwoFieldModel


class Serializer(ModelSerializer):
    class Meta:
        publish_fields = ('id',)
        update_fields = ('text', 'number')
        model = TwoFieldModel


class Router(BaseModelPublisherRouter):
    model = TwoFieldModel
    serializer_class = Serializer


class TestBaseModelPublisherRouter(DragonTestCase):
    def setUp(self):
        self.router = Router(self.connection)

    def test_created(self):
        data = {'text': 'text', 'number': 3}
        self.router.create(**data)
        model = self.router.model.objects.get()
        self.assertIsNotNone(model)

    def test_created_publish(self):
        self.router.subscribe(**{'channel': 'client-channel'})
        data = {'text': 'text', 'number': 3}
        self.router.create(**data)
        actual = self.connection.last_pub
        expected = {'action': 'created', 'channel': 'twofieldmodel|', 'data': {'_type': 'twofieldmodel', 'id': 1}}
        self.assertDictEqual(actual, expected)



from datetime import date
from decimal import Decimal
from django.db.models.fields.files import ImageFieldFile, FileField
from ..serializers.serializer_importer import get_serializer
from ..models import get_property


class BaseSerializer(object):
    _serializers_set_ = False

    base_channel = None
    publish_fields = None
    update_fields = None
    type_name = None
    id_field = 'id'

    def __init__(self, context=None, **kwargs):
        self.context = context
        self.kwargs = kwargs
        self._set_field_serializers()

    def _set_field_serializers(self):
        if self._serializers_set_:
            return
        if not hasattr(self.__class__, '_serializers'):
            setattr(self.__class__, '_serializers', [])
        serializers = [s for s in dir(self) if s.endswith('_serializer')]
        for serializer in serializers:
            ser = getattr(self, serializer)
            if isinstance(ser, str):
                ser = get_serializer(ser, self.__class__)
                setattr(self, serializer, ser)

            if ser not in self.__class__._serializers:
                self.__class__._serializers.append(ser)
        self._serializers_set_ = True

    @classmethod
    def get_base_channel(cls):
        return cls.base_channel

    @classmethod
    def _get_type_name(cls):
        return cls.type_name

    @classmethod
    def get_publish_fields(cls):
        return cls.publish_fields

    def get_update_fields(self):
        return self.update_fields

    def get_publish_channel(self):
        return self.base_channel

    def serialize(self, obj, ignore_fields=[]):
        publish_fields = self.get_publish_fields()
        if not publish_fields:
            raise Exception('no publish fields set')
        data = dict({f: self._serialize_value(obj, f) for f in publish_fields if f not in ignore_fields})
        return data

    def compose_updated_data(self, instance, updated_data):
        data = dict(updated_data)
        data['_type'] = self._get_type_name()
        data[self.id_field] = getattr(instance, self.id_field)
        return data

    def _serialize_value(self, obj, attr_name):
        obj_serializer = getattr(self, '{}_serializer'.format(attr_name), None)
        if obj_serializer:
            obj_serializer = obj_serializer()
        if hasattr(self, 'serialize_{}'.format(attr_name)):
            m = getattr(self, 'serialize_{}'.format(attr_name))
            return m(obj, serializer=obj_serializer)

        val = get_property(obj, attr_name)

        if obj_serializer and hasattr(val, 'all'):
            return [obj_serializer.serialize(o) for o in val.all()]
        elif obj_serializer:
            return obj_serializer.serialize(val)

        if isinstance(val, date):
            return str(val)
        if isinstance(val, Decimal):
            return str(val)
        if isinstance(val, ImageFieldFile) or isinstance(val, FileField):
            try:
                return val.url
            except:
                return None
        return val



from django.db import models
from ..serializers.model_serializer import ModelSerializer
from .dragon_test_case import DragonTestCase
from .models import SDModel


class FooM2M(SDModel):
    name = models.CharField(max_length=20)


class BarM2M(SDModel):
    foos = models.ManyToManyField(FooM2M)
    number = models.IntegerField()


class FooM2MSerializer(ModelSerializer):
    barm2m_set = 'BarM2MSerializer'
    class Meta:
        model = FooM2M
        update_fields = ('name', 'barm2m_set')


class BarM2MSerializer(ModelSerializer):
    foos = FooM2MSerializer

    class Meta:
        model = BarM2M
        update_fields = ('number', 'foos')


class TestModelSerializer(DragonTestCase):
    def test_deserialize_m2m(self):
        data = {
            'number': 3,
            'foos': [
                {'name': 'foo'}
            ]
        }
        serializer = BarM2MSerializer(data)
        bar = serializer.save()
        self.assertEqual(bar.number, data['number'])
        self.assertEqual(bar.foos.first().name, data['foos'][0]['name'])

    def test_deserialize_reverse_m2m(self):
        data = {
            'name': 'foo',
            'barm2m_set': [
                {'number': 3}
            ]
        }
        serializer = FooM2MSerializer(data)
        foo = serializer.save()
        self.assertEqual(foo.name, data['name'])
        self.assertEqual(foo.barm2m_set.first().number, data['barm2m_set'][0]['number'])



from ..pubsub_providers.base_provider import PUBACTIONS
from ..pubsub_providers.model_channel_builder import filter_channels_by_model
from ..pubsub_providers.publisher_factory import get_publisher


publisher = get_publisher()


def publish_model(model_instance, serializer, action, changed_fields=None):
    if action is PUBACTIONS.updated and not changed_fields:
        return

    # if hasattr(serializer, '_cache'):
    #     serializer.remove_from_cache()

    base_channel = serializer.get_base_channel()
    all_model_channels = publisher.get_channels(base_channel)
    channels = filter_channels_by_model(all_model_channels, model_instance)
    remove_from_channels = set(all_model_channels) - set(channels)

    if channels:
        publish_data = dict({'data': serializer.serialize(fields=changed_fields)})
        publish_data['action'] = action

        for c in channels:
            publish_data['channel'] = c
            publisher.publish(c, publish_data)

    if changed_fields:
        publish_data = {'data': {'id': model_instance.pk}}
        publish_data['action'] = PUBACTIONS.deleted
        for channel in remove_from_channels:
            publisher.publish(channel, publish_data)
